@@ -10,6 +10,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from openstack import exceptions
+from openstack.orchestration.util import template_utils
 from openstack.orchestration.v1 import resource as _resource
 from openstack.orchestration.v1 import software_config as _sc
 from openstack.orchestration.v1 import software_deployment as _sd
@@ -18,8 +20,6 @@ from openstack.orchestration.v1 import stack_environment as _stack_environment
 from openstack.orchestration.v1 import stack_files as _stack_files
 from openstack.orchestration.v1 import stack_template as _stack_template
 from openstack.orchestration.v1 import template as _template
-from openstack.orchestration.util import template_utils
-from openstack import exceptions
 from openstack import proxy
 from openstack import resource
 
@@ -487,13 +487,21 @@ class Proxy(proxy.Proxy):
             }
         }
 
-    def _service_cleanup(self, dry_run=True, status_queue=None):
+    def _service_cleanup(self, dry_run=True, client_status_queue=None,
+                         identified_resources=None,
+                         filters=None, resource_evaluation_fn=None):
         stacks = []
         for obj in self.stacks():
-            stacks.append(obj)
-            self._project_cleanup_del_res(
-                self.delete_stack, obj,
-                dry_run, status_queue)
+            need_delete = self._service_cleanup_del_res(
+                self.delete_stack,
+                obj,
+                dry_run=dry_run,
+                client_status_queue=client_status_queue,
+                identified_resources=identified_resources,
+                filters=filters,
+                resource_evaluation_fn=resource_evaluation_fn)
+            if not dry_run and need_delete:
+                stacks.append(obj)
 
         for stack in stacks:
             self.wait_for_delete(stack)
